package net.locusworks.s3sync.client;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.LinkedHashMap;
import java.util.Map;
import org.apache.commons.codec.digest.DigestUtils;
import net.locusworks.logger.ApplicationLogger;
import net.locusworks.logger.ApplicationLoggerFactory;

public class FileManager implements AutoCloseable {
  
  private ApplicationLogger logger = ApplicationLoggerFactory.getLogger(FileManager.class);
  
  public static final String FILE_CSV = "upload.csv";
  
  private Map<String, FileDetail> detailMap;
  
  private static FileManager instance;
  
  private FileManager() throws IOException {
    detailMap = new LinkedHashMap<String, FileDetail>();
    readFile();
  }

  private void readFile() throws IOException {
    Path file = Paths.get(FILE_CSV);
    if (Files.notExists(file)) return;
    
    try(BufferedReader reader = Files.newBufferedReader(file)) {
      String line = null;
      while((line = reader.readLine()) != null) {
        String[] values = line.split(",");
        if (values.length != 3) {
          logger.warn("Invalid entry detected: " + line);
          continue;
        }
        
        FileDetail fd = new FileDetail(values[0], values[1], Boolean.valueOf(values[2]));
        detailMap.put(values[0], fd);
      }
    }
  }
  
  private void saveFile() throws IOException {
    try(BufferedWriter writer = Files.newBufferedWriter(Paths.get(FILE_CSV))) {
      writer.write("FILE_NAME,HASH,STATUS\n");
      for(FileDetail v : detailMap.values()) {
        writer.write(String.format("%s,%s,%s%n", v.getFile(), v.getHash(), v.isUploaded()));
      };
    }
  }
  
  public boolean addEntry(String file, String hash, boolean uploaded) {
    return addEntry(new FileDetail(file, hash, uploaded));
  }
  
  public boolean addEntry(FileDetail fd) {
    return detailMap.put(fd.getFile(), fd) != null;
  }
  
  public FileDetail uploadFile(Path path) throws IOException {
    boolean newFile = false;
    String file = path.toString();
    FileDetail fd = null;
    if (detailMap.containsKey(file)) {
      fd = detailMap.get(file);
    } else {
      newFile = true;
      fd = new FileDetail(file, DigestUtils.sha1Hex(Files.newInputStream(path)), false);
    }
    
    String sha1 = newFile ? fd.getHash() : DigestUtils.sha1Hex(Files.newInputStream(path));
    
    if (!sha1.equals(fd.getHash())) {
      fd.setUploaded(false);
    }
    
    return fd;
  }

  @Override
  public void close() throws Exception {
    saveFile();
  }
  
  public static FileManager getInstance() throws IOException {
    if (instance == null) {
      instance = new FileManager();
    }
    return instance;
  }
  
}
