package net.locusworks.s3sync.client;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import com.amazonaws.AmazonClientException;
import com.amazonaws.regions.Regions;
import com.amazonaws.services.s3.AmazonS3;
import com.amazonaws.services.s3.AmazonS3ClientBuilder;
import com.amazonaws.services.s3.model.Bucket;
import com.amazonaws.services.s3.transfer.TransferManager;
import com.amazonaws.services.s3.transfer.TransferManagerBuilder;
import com.amazonaws.services.s3.transfer.Upload;
import net.locusworks.logger.ApplicationLogger;
import net.locusworks.logger.ApplicationLoggerFactory;
import net.locusworks.s3sync.conf.ConfigurationManager;

public class S3Client {

  private ApplicationLogger logger = ApplicationLoggerFactory.getLogger(S3Client.class);

  private AmazonS3 s3Client;
  private String bucket;
  private Path syncFolder;

  public S3Client(ConfigurationManager conf) {
    String region = conf.getRegion();
    this.s3Client = AmazonS3ClientBuilder.standard().withRegion(Regions.fromName(region)).build();
    Bucket bucket = s3Client.listBuckets().stream()
        .filter(b -> b.getName().equals(conf.getBucketName())).findFirst().orElse(null);

    if (bucket == null) {
      logger.error("Unable to find bucket with name: %s%nExiting", conf.getBucketName());
      System.exit(-1);
    }
    logger.info("Found Bucket: %s", bucket);
    this.bucket = conf.getBucketName();
    this.syncFolder = conf.getSyncFolder();
  }

  public void uploadFile(Path file) {
    TransferManager xferMgr = TransferManagerBuilder.standard().withS3Client(s3Client).build();
    uploadFile(xferMgr, file);
    xferMgr.shutdownNow();
  }

  public void uploadFile(TransferManager xferMgr, Path file) {
    boolean xferMgrNull = xferMgr == null;
    xferMgr = !xferMgrNull ? xferMgr : TransferManagerBuilder.standard().withS3Client(s3Client).build();
    FileDetail fd = null;
    try {
      fd = FileManager.getInstance().uploadFile(file);
      if (fd.isUploaded()) return;
      logger.info("Uploading file: %s", file);
      Upload xfer = xferMgr.upload(bucket, getPath(file), file.toFile());
      xfer.waitForCompletion();
      fd.setUploaded(true);
      FileManager.getInstance().addEntry(fd);
      logger.info("Done uploading %s", file);
    } catch (AmazonClientException | InterruptedException | IOException e) {
      if (fd != null) {
        fd.setUploaded(false);
        try {
          FileManager.getInstance().addEntry(fd);
        } catch (IOException e1) {
          logger.error("Unable to save file to file manager: " + e1.getMessage());
        }
      }
      logger.error(e.getMessage());
    } finally {
      if (xferMgrNull) {
        xferMgr.shutdownNow();
      }
    }
  }

  public void syncFolder() throws IOException {
    TransferManager xferMgr = TransferManagerBuilder.standard().withS3Client(s3Client).build();
    try (FileManager manager = FileManager.getInstance()) {
      Files.walk(syncFolder)
      .filter(f -> Files.isRegularFile(f))
      .forEach(f -> uploadFile(xferMgr, syncFolder.resolve(f)));
    } catch (Exception e) {
      logger.error("Unable to load file Manager: " + e.getMessage());
    } 

    xferMgr.shutdownNow();
  }
 
  private String getPath(Path file) {
    if (file.getParent() == null) return file.getFileName().toString();
    Path relative = syncFolder.relativize(file);
    return relative.toString().replace("\\", "/");
  }

}
